using System;
using System.Linq;
using System.Collections.Generic;

namespace Xamarin.Forms
{
	/// <summary>
	/// Simple wrap layout panel
	/// </summary>
	public class SimpleWrapLayout : Layout<View>
	{
		int Spacing  = 6;

		protected override SizeRequest OnSizeRequest (double widthConstraint, double heightConstraint)
		{

			double width = 0;
			double height = 0;
			double minWidth = 0;
			double minHeight = 0;
			double totalWidth = 0;
			int totalRows = 1;

			if (WidthRequest > 0)
				widthConstraint = Math.Min (widthConstraint, WidthRequest);

			double w =  Math.Max (0, widthConstraint);

			if (HeightRequest > 0)
				heightConstraint = Math.Min (heightConstraint, HeightRequest);

			double h =  Math.Max (0, heightConstraint);

			foreach (var item in Children)    
			{
				var size = item.GetSizeRequest(w, h);
				height = Math.Max (height, size.Request.Height);

				var calcWidth = width + size.Request.Width + Spacing;
				if (calcWidth > w) {
					totalRows++;
					totalWidth = Math.Max(width, totalWidth);
					width = size.Request.Width;
				} else
					width = calcWidth;

				minHeight = Math.Max(minHeight, size.Minimum.Height);
				minWidth = Math.Max (minWidth, size.Minimum.Width);
			}

			if (totalRows > 1) {
				width = Math.Max(width, totalWidth);
				height = (height + Spacing) * totalRows - Spacing; 
			}

			return new SizeRequest(new Size(width, height), new Size(minWidth,minHeight));
		}

		protected override void LayoutChildren (double x, double y, double width, double height)
		{

			double yPos = y, xPos = x, rowHeight = 0;

			foreach (var child in Children.Where(c => c.IsVisible)) 
			{
				var request = child.GetSizeRequest (width, height);

				double itemWidth = request.Request.Width;
				double itemHeight = request.Request.Height;
				rowHeight = Math.Max(rowHeight, itemHeight);

				if (xPos + itemWidth > width) {
					xPos = x;
					yPos += rowHeight + Spacing;
					rowHeight = 0;
				}

				var region = new Rectangle (xPos, yPos, itemWidth, itemHeight);
				LayoutChildIntoBoundingRegion (child, region);
				xPos += region.Width + Spacing;
			}
		}
	}
}